###################################################
### chunk number 1: preliminaries
###################################################
library("FME")
options(prompt = "> ")
options(width=70)


###################################################
### chunk number 2: 
###################################################
require(FME)


###################################################
### chunk number 3: 
###################################################
Obs <- data.frame(x=c(   28,  55,   83,  110,  138,  225,  375),   # mg COD/l
                  y=c(0.053,0.06,0.112,0.105,0.099,0.122,0.125))   # 1/hour


###################################################
### chunk number 4: 
###################################################
Model <- function(p,x) return(data.frame(x=x,y=p[1]*x/(x+p[2])))


###################################################
### chunk number 5: 
###################################################
Residuals  <- function(p) (Obs$y-Model(p,Obs$x)$y)


###################################################
### chunk number 6: 
###################################################
print(system.time(
P      <- modFit(f=Residuals,p=c(0.1,1))
))


###################################################
### chunk number 7: 
###################################################
sP    <- summary(P)
sP


###################################################
### chunk number 8: 
###################################################
x      <-0:375


###################################################
### chunk number 9: Monplot
###################################################
par(mfrow=c(2,2))
plot(P,mfrow=NULL)
plot(Obs,pch=16,cex=2,xlim=c(0,400),ylim=c(0,0.15),
     xlab="mg COD/l",ylab="1/hr",main="best-fit")
lines(Model(P$par,x))
par(mfrow=c(1,1))


###################################################
### chunk number 10: Monplot
###################################################
par(mfrow=c(2,2))
plot(P,mfrow=NULL)
plot(Obs,pch=16,cex=2,xlim=c(0,400),ylim=c(0,0.15),
     xlab="mg COD/l",ylab="1/hr",main="best-fit")
lines(Model(P$par,x))
par(mfrow=c(1,1))


###################################################
### chunk number 11: 
###################################################
Covar   <- sP$cov.scaled * 2.4^2/2
s2prior <- sP$modVariance

print(system.time(
MCMC <- modMCMC(f=Residuals,p=P$par,jump=Covar,niter=3000,
                var0=s2prior,wvar0=1,lower=c(0,0))
))


###################################################
### chunk number 12: 
###################################################
print(system.time(
MCMC <- modMCMC(f=Residuals,p=P$par,jump=Covar,niter=3000, ntrydr=3,
                var0=s2prior,wvar0=1,updatecov=100,lower=c(0,0))
))
MCMC$count


###################################################
### chunk number 13: Monmcmc
###################################################
plot(MCMC,Full=TRUE)


###################################################
### chunk number 14: Monmcmc
###################################################
plot(MCMC,Full=TRUE)


###################################################
### chunk number 15: Monhist
###################################################
hist(MCMC,Full=TRUE,col="darkblue")


###################################################
### chunk number 16: Monhist
###################################################
hist(MCMC,Full=TRUE,col="darkblue")


###################################################
### chunk number 17: Monpairs
###################################################
pairs(MCMC)


###################################################
### chunk number 18: Monpairs
###################################################
pairs(MCMC)


###################################################
### chunk number 19: 
###################################################
cor(MCMC$pars)
cov(MCMC$pars)
sP$cov.scaled


###################################################
### chunk number 20: 
###################################################
MC <- as.mcmc(MCMC$pars)
raftery.diag(MC)


###################################################
### chunk number 21: cumu
###################################################
cumuplot(MC)


###################################################
### chunk number 22: cumu
###################################################
cumuplot(MC)


###################################################
### chunk number 23: 
###################################################
sR<-sensRange(parInput=MCMC$pars,func=Model,x=1:375)


###################################################
### chunk number 24: Monsum
###################################################
plot(summary(sR),quant=TRUE)
points(Obs)


###################################################
### chunk number 25: Monsum
###################################################
plot(summary(sR),quant=TRUE)
points(Obs)


###################################################
### chunk number 26: 
###################################################
pset <- attributes(sR)$pset


###################################################
### chunk number 27: 
###################################################
nout  <- nrow(sR)
sR2   <- sR
ivar  <- 3:ncol(sR)
error <- rnorm(nout,mean=0,sd=sqrt(MCMC$sig[pset]))
sR2[,ivar] <- sR2[,ivar] + error


###################################################
### chunk number 28: MonsumM
###################################################
plot(summary(sR2),quant=TRUE)
points(Obs)


###################################################
### chunk number 29: MonsumM
###################################################
plot(summary(sR2),quant=TRUE)
points(Obs)


